/*
 * xdraw.c -- icebuteo
 *
 * $Revision: 16 $
 * $Date: 2006-07-22 21:36:22 +0200 (Sat, 22 Jul 2006) $
 * 
 * Copyright (C) 2006  Josef Kufner <jk@myserver.cz>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#include <assert.h>
#include <string.h>
#include <stdarg.h>
#include "icebuteo.h"
#include "xdraw.h"

#define DEBUG 0
#include "debug.h"


/*****************************************************************************
 * xdraw_frame -- draw frame
 * 
 * Args:
 *   x, y	... top left corner
 *   w, h	... width, height
 *   is_raised	... shadow type
 */
void xdraw_frame(XWINDOW *xwin, long x, long y, long w, long h, int is_raised) {
	assert(xwin);
	assert(is_raised == 0 || is_raised == 1);
	
	xdraw_frame_color(xwin, x, y, w, h, is_raised,
			is_raised ? C_SH_L : C_SH_D,
			is_raised ? C_SH_D : C_SH_L,
			0);
	return;
}


/*****************************************************************************
 * xdraw_frame -- draw frame
 * 
 * Args:
 *   x, y	... top left corner
 *   w, h	... width, height
 *   is_raised	... shadow type
 *   cl, cs	... color of light, shadow
 */
void xdraw_frame_color(XWINDOW *xwin, long x, long y, long w, long h, int is_raised, int cl, int cs, int draw_border) {
	assert(xwin);
	assert(is_raised == 0 || is_raised == 1);
	assert(cl >= 0 && cl < C_MAX);
	assert(cs >= 0 && cs < C_MAX);
	
	XSetForeground(xwin->disp, xwin->gc, xwin->color[cl].pixel);
	XDrawLine(xwin->disp, xwin->win, xwin->gc, x, y, x + 0, y + h - 1);
	XDrawLine(xwin->disp, xwin->win, xwin->gc, x, y, x + w - 1, y);
	XSetForeground(xwin->disp, xwin->gc, xwin->color[cs].pixel);
	XDrawLine(xwin->disp, xwin->win, xwin->gc, x + 1, y + h - 1, x + w - 1, y + h - 1);
	XDrawLine(xwin->disp, xwin->win, xwin->gc, x + w - 1, y + 1, x + w - 1, y + h - 1);
	if (draw_border) {
		XSetForeground(xwin->disp, xwin->gc, xwin->color[C_BORDER].pixel);
		XDrawLine(xwin->disp, xwin->win, xwin->gc, x, y + h, x + w, y + h);
		XDrawLine(xwin->disp, xwin->win, xwin->gc, x + w, y, x + w, y + h);
	}
	return;
}


/*****************************************************************************
 * xdraw_select_font -- select font for xdraw_text
 */
int xdraw_select_font(XWINDOW *xwin, int fid) {
	assert(xwin != NULL);
	assert(fid >= 0 && fid < FONT_MAX);

	if (!xwin->font[fid]) {
		DBG_MSG(4, "Loading font \"%s\"", cfg.font[fid]);
		if ((xwin->font[fid] = XLoadQueryFont(xwin->disp, cfg.font[fid])) == NULL) {
			fprintf(stderr, PROG_NAME ": Cannot open font \"%s\"\n", cfg.font[fid]);
			xwin->is_font_selected = 0;
			return 0;
		}
	}
	XSetFont(xwin->disp, xwin->gc, xwin->font[fid]->fid);
	xwin->act_fid = fid;
	xwin->is_font_selected = 1;
	return 1;
}


/*****************************************************************************
 * xdraw_text -- draw text
 * 
 * Args:
 *   x, y	... baseline edge
 *   text	... text string
 *   align	... ref. point position (0 = left aligned text, 1 = right)
 *   shadow	... shadow? and how far?
 */
void xdraw_text(XWINDOW *xwin, long x, long y, char* text, float align, int shadow) {
	int offset = 0;
	assert(xwin != NULL);
	assert(xwin->is_font_selected);
	assert(text != NULL);
	assert(xwin->font);
	assert(shadow >= 0);
	assert(align >= 0.0 && align <= 1.0);
	
	XSetBackground(xwin->disp, xwin->gc, xwin->color[C_BG].pixel);
	if (align != 0) {
		offset = (int)(align * XTextWidth(xwin->font[xwin->act_fid], text, strlen(text)));
	}
	if (shadow) {
		XSetForeground(xwin->disp, xwin->gc, xwin->color[C_DARK].pixel);
		XDrawString(xwin->disp, xwin->win, xwin->gc, x - offset + shadow, y + shadow, text, strlen(text));
		XSetForeground(xwin->disp, xwin->gc, xwin->color[C_TEXT].pixel);
		XDrawString(xwin->disp, xwin->win, xwin->gc, x - offset, y, text, strlen(text));
	} else {
		XSetForeground(xwin->disp, xwin->gc, xwin->color[C_TEXT].pixel);
		XDrawString(xwin->disp, xwin->win, xwin->gc, x - offset, y, text, strlen(text));
	}
}


/*****************************************************************************
 * xdraw_bar -- draw progress bar
 *
 * Args:
 *    x, y	... left top corner
 *    w, h	... width, height
 *    fill	... size of filled part (0.0 - 1.0)
 */
void xdraw_bar(XWINDOW *xwin, long x, long y, long w, long h, float fill) {
	long iw;
	assert(xwin != NULL);
	assert(w >= 0);
	assert(h >= 0);
	assert(fill >= 0.0 && fill <= 1.0);

	if (w < 8)
		w = 8;
	if (h < 8)
		h = 8;
	iw = (w - 7) * fill;

	/* frame around */
	xdraw_frame(xwin, x, y, w, h, 0);
	
	/* background */
	XSetForeground(xwin->disp, xwin->gc, xwin->color[C_DARK].pixel);
	XFillRectangle(xwin->disp, xwin->win, xwin->gc, x + 1, y + 1, w - 2, h - 2);

	if (iw) {
		/* filled part's frame */
		xdraw_frame_color(xwin, x + 2, y + 2, iw + 2, h - 5, 1, C_BAR_L, C_BAR_S, 1);

		/* fill */
		XSetForeground(xwin->disp, xwin->gc, xwin->color[C_BAR].pixel);
		XFillRectangle(xwin->disp, xwin->win, xwin->gc, x + 3, y + 3, iw, h - 7);
	}
}


/*****************************************************************************
 * xdraw_vsep -- Draw vertical separator (line)
 */
void xdraw_vsep(XWINDOW *xwin, long x, long y, long len) {
	assert(xwin != NULL);
	assert(len > 0);
	
	len--;
	XSetForeground(xwin->disp, xwin->gc, xwin->color[C_SH_D].pixel);
	XDrawLine(xwin->disp, xwin->win, xwin->gc, x, y, x, y + len);
	XDrawPoint(xwin->disp, xwin->win, xwin->gc, x + 1, y);
	XSetForeground(xwin->disp, xwin->gc, xwin->color[C_SH_L].pixel);
	XDrawLine(xwin->disp, xwin->win, xwin->gc, x + 1, y + 1, x + 1, y + len);
	XDrawPoint(xwin->disp, xwin->win, xwin->gc, x, y + len);
}


/*****************************************************************************
 * xdraw_hsep -- Draw horizontal separator (line)
 */
void xdraw_hsep(XWINDOW *xwin, long x, long y, long len) {
	assert(xwin != NULL);
	assert(len > 0);

	len--;
	XSetForeground(xwin->disp, xwin->gc, xwin->color[C_SH_D].pixel);
	XDrawLine(xwin->disp, xwin->win, xwin->gc, x, y, x + len, y);
	XDrawPoint(xwin->disp, xwin->win, xwin->gc, x, y + 1);
	XSetForeground(xwin->disp, xwin->gc, xwin->color[C_SH_L].pixel);
	XDrawLine(xwin->disp, xwin->win, xwin->gc, x + 1, y + 1, x + len, y + 1);
	XDrawPoint(xwin->disp, xwin->win, xwin->gc, x + len, y);
}
