/*
 * xwindow.c -- icebuteo
 *
 * $Revision: 16 $
 * $Date: 2006-07-22 21:36:22 +0200 (Sat, 22 Jul 2006) $
 * 
 * Copyright (C) 2006  Josef Kufner <jk@myserver.cz>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "icebuteo.h"
#include "xwindow.h"
#include "xwindow-lex.h"
#include "xdraw.h"

#define DEBUG 0
#include "debug.h"

#define MAX_LINE_LEN	1024

static const int default_color[C_MAX][3] = {
	/* C_BG     */ { 0xde, 0xdb, 0xd6 },
	/* C_DARK   */ { 0xc6, 0xc3, 0xbd },
	/* C_FG     */ { 0x00, 0x00, 0x00 },
	/* C_TEXT   */ { 0x00, 0x00, 0x00 },
	/* C_SH_L   */ { 0xff, 0xfb, 0xff },
	/* C_SH_D   */ { 0x9c, 0x9a, 0x94 },
	/* C_BAR    */ { 0x4a, 0x69, 0x86 },
	/* C_BAR_L  */ { 0x5a, 0x8a, 0xb5 },
	/* C_BAR_S  */ { 0x39, 0x49, 0x52 },
	/* C_BORDER */ { 0x00, 0x00, 0x00 }
};

extern int the_time;


/*****************************************************************************
 * xwindow_init -- create window
 */
XWINDOW * xwindow_init(void) {
	XWINDOW *w;
	XSetWindowAttributes attributes;
	int i;

	if ((w = (XWINDOW *) malloc(sizeof(XWINDOW)))) {
		/* init vars */
		w->is_broken = 0;
		w->can_redraw = 0;
		w->is_font_selected = 0;
		for (i = 0; i < FONT_MAX; i++)
			w->font[i] = NULL;
		w->widget = NULL;
		w->column = NULL;
		w->widget_count = 0;
		w->column_count = 0;

		/* display */
		if (!(w->disp = XOpenDisplay(NULL))) {
			fprintf(stderr, PROG_NAME ": Can't open display.\n");
			xwindow_destroy(w);
			return NULL;
		}
		
		/* colors */
		for(i = 0; i < C_MAX; i++) {
			w->color[i].pixel = i + 1;
			w->color[i].red   = default_color[i][0] << 8;
			w->color[i].green = default_color[i][1] << 8;
			w->color[i].blue  = default_color[i][2] << 8;
			XAllocColor(w->disp, DefaultColormap(w->disp, DefaultScreen(w->disp)), w->color + i);
		}
		
		/* window */
		attributes.background_pixel      = w->color[C_BG].pixel;
		attributes.border_pixel          = w->color[C_BORDER].pixel;
		attributes.override_redirect     = 1; 
		w->width = 1;
		w->height = 1;
		w->win = XCreateWindow(w->disp, DefaultRootWindow(w->disp),
				0, 0, w->width, w->height, 1,
				CopyFromParent, 0, CopyFromParent, 
				CWBackPixel | CWBorderPixel | CWOverrideRedirect,
				&attributes);
		XChangeProperty(w->disp, w->win, XA_WM_NAME, XA_STRING, 8,
				PropModeReplace, (const unsigned char *) cfg.win_title,
				strlen(PROG_NAME));
		XSelectInput(w->disp, w->win, ExposureMask | StructureNotifyMask);
		XMapWindow(w->disp, w->win);
		XClearWindow(w->disp, w->win);
		XFlush(w->disp);
		return w;
	} else {
		return NULL; 
	}
}


/*****************************************************************************
 * xwindow_load_widgets -- load widget configuration from file
 */
void xwindow_load_widgets(XWINDOW *xwin, char *filename) {
	int i;

	assert(xwin != NULL);
	assert(filename != NULL);
	assert(xwin->widget == NULL);  /* load only once */
	assert(xwin->column == NULL);  /* load only once */
	assert(cfg.max_widgets > 0);
	assert(cfg.max_columns > 0);

	DBG_MSG(3, "Loading widgets...");
	
	/* initialize columns */
	xwin->column_count = cfg.max_columns;
	if (!(xwin->column = (COLUMN *) malloc(sizeof(COLUMN) * xwin->column_count))) {
		perror(PROG_NAME ": load widgets - column");
		exit(1);
	}
	for (i = 0; i < xwin->column_count; i++)
		xwin->column[i].not_empty = 0;
	
	/* initialize widgets */
	xwin->widget_count = cfg.max_widgets;
	if (!(xwin->widget = (WIDGET *) malloc(sizeof(WIDGET) * xwin->widget_count))) {
		perror(PROG_NAME ": load widgets - widget");
		exit(1);
	}
	for (i = 0; i < xwin->widget_count; i++)
		xwin->widget[i].type = WID_NONE;
	
	/* load widgets */
	if ((xwindow_lex_in = fopen(cfg.layout_file, "r"))) {
		xwindow_lexscan(xwin);
		fclose(xwindow_lex_in);
	} else {
		perror(filename);
		exit(1);
	}
	
	DBG_MSG(3, "Loading widgets... Done.");
	xwindow_recalculate(xwin);
}


/*****************************************************************************
 * xwindow_destroy -- clean up
 */
void xwindow_destroy(XWINDOW *xwin) {
	assert(xwin != NULL);
	if (xwin->disp)
		XCloseDisplay(xwin->disp);
	if (xwin)
		free(xwin);
	xwin = NULL;
}


/*****************************************************************************
 * xwindow_process_events -- process all pending events
 */
void xwindow_process_events(XWINDOW *xwin) {
	XEvent event;
	assert(xwin != NULL);
	
	DBG_MSG(4, "Processing events...");
	while (XPending(xwin->disp)) {
		XNextEvent(xwin->disp, &event);
		switch(event.type) {
			case MapNotify:
				DBG_MSG(5, "Event: MapNotify");
				xwindow_reposition (xwin, 0);
				xwin->gc = XCreateGC(xwin->disp, xwin->win, 0, 0);
				xwin->can_redraw = 1;
				xwindow_recalculate(xwin);
				break;

			case Expose:
				DBG_MSG(5, "Event: Expose");
				while (XCheckTypedEvent(xwin->disp, Expose, &event))
					;
				xwindow_redraw(xwin, 0);
				break;

			case CirculateNotify:
				DBG_MSG(5, "Event: CirculateNotify");
				break;

			case ConfigureNotify:
				DBG_MSG(5, "Event: ConfigureNotify");
				break;

			case GravityNotify:
				DBG_MSG(5, "Event: GravityNotify");
				break;

			case ReparentNotify:
				DBG_MSG(5, "Event: ReparentNotify");
				break;

			case UnmapNotify:
				DBG_MSG(5, "Event: UnmapNotify");
				break;

			case DestroyNotify:
				DBG_MSG(5, "Event: DestroyNotify");
				xwin->is_broken = 1;
				xwin->can_redraw = 0;
				break;
				
			default:
				DBG_MSG(5, "Event: unknown");
		}
	}
	DBG_MSG(4, "All events done.");
}


/*****************************************************************************
 * xwindow_reposition
 */
void xwindow_reposition (XWINDOW *xwin, int now) {
	XWindowAttributes win_attr;

	assert(xwin != NULL);

	if (xwin->width > 0 && xwin->height > 0) {
		XGetWindowAttributes(xwin->disp, DefaultRootWindow(xwin->disp), &win_attr);
		xwin->root_width = win_attr.width;
		xwin->root_height = win_attr.height;
		xwin->x = (xwin->root_width - xwin->width) >> 1;
		xwin->y = 0;
		XMoveResizeWindow(xwin->disp, xwin->win, xwin->x, xwin->y, xwin->width, xwin->height);
	} else {
	}
	if (now) {
		XFlush(xwin->disp);
	}
}


/*****************************************************************************
 * xwindow_recalculate -- recalulate size of window and columns
 */
void xwindow_recalculate(XWINDOW *xwin) {
	int i;
	int old_height, old_width;
	assert(xwin != NULL);
	assert(xwin->column != NULL);
	assert(xwin->column_count >= 0);

	DBG_MSG(6, "Recalculating window...");
	
	/* calculate window size and columns positions */
	old_width = xwin->width;
	old_height = xwin->height;
	xwin->width = 0;
	xwin->height = 0;
	for (i = 0; i < xwin->column_count; i++) {
		DBG_MSG(6, "Column %d: not_empty = %d, vis. var[%d] = %d", i, xwin->column[i].not_empty,
				xwin->column[i].visibility_var_id,
				var_get_as_int(xwin->column[i].visibility_var_id));
		
		if (xwin->column[i].not_empty && var_get_as_int(xwin->column[i].visibility_var_id)) {
			DBG_MSG(6, "Column %d visible.", i);
			xwin->column[i].visible = 1;
			xwin->column[i].x = xwin->width;
			xwin->column[i].y = 0;
			xwin->width += xwin->column[i].width;
			if (xwin->height < xwin->column[i].height)
				xwin->height = xwin->column[i].height;
		} else {
			DBG_MSG(7, "Column %d ignored.", i);
			xwin->column[i].visible = 0;
		}
	}

	if (old_height != xwin->height || old_width != xwin->width) {
		/* window's size has changed -- so, resize it */
		xwindow_reposition(xwin, 1);
	}
	
	DBG_MSG(6, "Recalculating window... Done.");
}


/*****************************************************************************
 * xwindow_redraw -- redraw entire window
 */
void xwindow_redraw(XWINDOW *xwin, int recalculate) {
	//char str[32];
	int c, i;
	long x, y;
	int wid_text_shadow = 0;
	float wid_text_align = 0.0;
	COLUMN *col;
	WIDGET *wid;
	assert(xwin != NULL);
	DBG_MSG(4, "Redrawing...");
	
	if (xwin->can_redraw && !xwin->is_broken) {
		if (recalculate)
			xwindow_recalculate(xwin);
		XClearWindow(xwin->disp, xwin->win);
		
		/* draw columns */
		for (c = 0; c < xwin->column_count; c++) {
			if (xwin->column[c].visible) {
				col = &(xwin->column[c]);
				xdraw_frame(xwin, col->x, col->y, col->width, xwin->height, 1);
			}
		}
		/* draw widgets */
		xdraw_select_font(xwin, FONT_NORMAL);
		for (i = 0; i < xwin->widget_count; i++) {
			wid = &(xwin->widget[i]);
			if (wid->type != WID_NONE && xwin->column[wid->column].visible) {
				col = &(xwin->column[wid->column]);
				x = wid->x + col->x;
				y = wid->y + col->y;
				switch(wid->type) {
					case WID_FRAME:
						xdraw_frame(xwin, x, y, wid->width, wid->height, 0);
						break;

					case WID_VSEP:
						xdraw_vsep(xwin, x, y, wid->height);
						break;

					case WID_HSEP:
						xdraw_hsep(xwin, x, y, wid->width);
						break;

					case WID_TEXT_DEFAULT:
						xdraw_select_font(xwin, FONT_NORMAL);
						wid_text_align = 0.0;
						wid_text_shadow = 0;
						break;
						
					case WID_TEXT_FONT:
						xdraw_select_font(xwin, var_get_as_int(wid->var_ref));
						break;

					case WID_TEXT_ALIGN:
						wid_text_align = var_get_as_float(wid->var_ref);
						break;

					case WID_TEXT_SHADOW:
						wid_text_shadow = var_get_as_int(wid->var_ref);
						break;

					case WID_TEXT:
						DBG_MSG(6, "Drawing text: \"%s\" at [ %ld, %ld ]",
								var_get_as_string(wid->var_ref),
								x, y);
						xdraw_text(xwin, x, y, var_get_as_string(wid->var_ref),
								wid_text_align, wid_text_shadow);
						break;

					case WID_BAR:
						xdraw_bar(xwin, x, y, wid->width, wid->height,
								var_get_as_float(wid->var_ref));
				}
			}
		}

		/*
		xdraw_select_font(xwin, FONT_BIG);
		xdraw_text(xwin, 200, 26, "Hello !", 0.5, 2);
		
		xdraw_select_font(xwin, FONT_NORMAL);
		sprintf(str, "%d %%", the_time % 100);
		xdraw_text(xwin, 150, 46, "Hello:", 0.0, 0);
		xdraw_text(xwin, 250, 46, str, 1.0, 0);
		xdraw_bar(xwin, 150, 50, 100, 0, (the_time % 100) / 100.0);
		
		sprintf(str, "%d g", 100 - (the_time % 100));
		xdraw_text(xwin, 150, 76, "World:", 0.0, 0);
		xdraw_text(xwin, 250, 76, str, 1.0, 0);
		xdraw_bar(xwin, 150, 80, 100, 0, (100 - (the_time % 100)) / 100.0);
		*/

		XFlush(xwin->disp);
	}
}
